#!/usr/bin/ruby

require 'GECS'

# Number of iterations expected
N = 1000

# Bootstrap or quick means
boot = true

# Experiment ID
xid = 0

# Retrieve perf stat data series as arrays of strings.
def getperfstatvar(filecontent,field)
  temp = filecontent.scan(/(\d*\.?\d*) +#{field}/).flatten
  raise "Count mismatch" if temp.length != N
  temp
end

# Retrieve perf report event counts as arrays of strings.
# Event count (approx.): 21652
# With -c 1000000 these should arrive at 2992.5 Hz.
def getperfrecvar(filecontent)
  temp = filecontent.scan(/Event count \(approx\.\): (\d*)/).flatten
  raise "Count mismatch" if temp.length != N
  temp
end

# Retrieve bash time data series as arrays of strings.
def getbashvar(filecontent,fieldnum)
  temp = filecontent.scan(/.*\n/).map(&:split).map{|x| x.fetch(fieldnum)}
  raise "Count mismatch" if temp.length != N
  temp
end

# Retrieve all data for one level of workload.
# Convert all units to seconds and floats for simplicity.
def gettreatment(os,workload)
  subdir = os==0 ? "14.0/" : "14.1/"
  perfstatfile = File.open(subdir + "log" + workload.to_s + "perfstat.txt").read
  perfrecfile = File.open(subdir + "log" + workload.to_s + "perfrec.txt").read
  bashfile = File.open(subdir + "log" + workload.to_s + "bash.txt").read
  results = Array.new(8)
  for i in 0..2
    results[i] = getbashvar(bashfile,i).map(&:to_f)
  end
  perfstatfields = ["seconds time elapsed","cpu-cycles","cpu-clock","task-clock"]
  for i in 3..6
    temp = getperfstatvar(perfstatfile,perfstatfields[i-3]).map!(&:to_f)
    case i
    when 4    # cycles
      temp.map!{|x| x/2992500000.0}
    when 5,6  # ms
      temp.map!{|x| x/1000.0}
    end
    results[i] = temp
  end
  results[7] = getperfrecvar(perfrecfile).map!{|x| x.to_f/2992.5}
  results
end

Experiment = GECS::Experiment.new(xid,
  ["os", "workload"],
  ["bash-elapsed", "bash-user", "bash-sys",
   "perf-elapsed", "perf-cpu-cycles", "perf-cpu-clock", "perf-task-clock",
   "perf-samples"],
  "Validation and selection of metrics for screening experiment to follow")

results = Hash.new
for os in 0..1
  for workload in 0..4
    results[GECS::Key.new(xid,[os,workload])] = gettreatment(os,workload)
  end
end

bag = GECS.newBag([Experiment],results)
if boot
  GECS.bootstrapMeans(bag,xid,
    [0.0001,0.0001,0.0001,0.0001,0.0001,0.0001,0.0001,0.0001])
else
  GECS.quickMeans(bag,xid)
end
File.open("0-data.txt","w") << GECS::dumpData(bag,xid)
File.open("0-parms.txt","w") << GECS::dumpParms(bag,xid,boot)
GECS.save("screening.gecs",bag)
