library(plotrix)

data  <- read.table("0-data.txt",header=T,colClasses="numeric")
parms <- read.table("0-parms.txt",header=T,colClasses="numeric")

# Get rid of the NA rows right away.
parms <- parms[!is.na(parms$workload) & !is.na(parms$os),]

numfacs <- 2
numdepvars <- ncol(data)-numfacs

# If T then the estimated attained precision of the bootstrap intervals is
# used to put error bars on the error bars.  Once it has been established
# that the precision is below the plot resolution, this is no longer
# desirable.
doublebars <- F

workloads <- unique(data$workload)
numWorkloads <- length(workloads)

# Allocate depvars to plots for each workload.
# These plot numbers restart at 1 for each row.
# The first row is for workload 0 and so on.
# Even skipped workloads need a row here.
colmap <- matrix(c(1,1,2,1,1,1,1,1,
		   1,2,3,1,2,2,2,2,
		   1,2,3,1,2,2,2,2,
		   1,2,1,1,1,1,1,1,
		   1,1,2,1,1,1,1,1),ncol=numdepvars,byrow=T)
maxCols <- max(colmap)

pdf("means-merged.pdf",pointsize=11)
cex <- 1.2
cexaxis <- 1.1
staxcex <- cexaxis * 2/3
layout(matrix(data=c(1,1,2,3,4,5,6,7,8,9,9,10,11,11,12),
  nrow=numWorkloads,ncol=maxCols,byrow=T),widths=c(2,4,1))

for (workload in workloads) {
  plotsThisRow <- max(colmap[workload+1,])

  if (T) {
    # Find the largest range for any plot pertaining to this workload.
    range <- 0
    for (plotnum in 1:plotsThisRow) {
      colfilter <- colmap[workload+1,]==plotnum
      min <- min(parms[parms$workload==workload,seq(numfacs+2,ncol(parms),4)[colfilter]])
      max <- max(parms[parms$workload==workload,seq(numfacs+3,ncol(parms),4)[colfilter]])
      range <- max(range,max-min)
    }
    cat(sprintf("Workload %d max range %f\n",workload,range))
  } else {
    # Or just hard-code a fixed value to make all plots the same scale.
    range <- 1.439469
  }

  # Draw all of the plots for this workload to that scale.
  for (plotnum in 1:plotsThisRow) {
    colfilter <- colmap[workload+1,]==plotnum
    topx  <- sum(colfilter)
    x     <- 1:topx
    xx    <- rep(x,each=2)
    means <- unlist(parms[parms$workload==workload,seq(numfacs+1,ncol(parms),4)[colfilter]])
    mins  <- unlist(parms[parms$workload==workload,seq(numfacs+2,ncol(parms),4)[colfilter]])
    maxs  <- unlist(parms[parms$workload==workload,seq(numfacs+3,ncol(parms),4)[colfilter]])
    precs <- unlist(parms[parms$workload==workload,seq(numfacs+4,ncol(parms),4)[colfilter]])
    oss   <- rep(unlist(parms[parms$workload==workload,1]),topx)
    cols  <- sapply(oss,function(x){if(x==0) "blue" else "red"})
    min   <- min(mins)
    max   <- max(maxs)
    adj   <- 0.25
    xlim  <- c(1-adj,topx+adj)
    adj   <- (range+min-max)/2
    ylim  <- if (adj<min) c(min-adj,max+adj) else c(0,range)
    par(mar=c(2.5,if (plotnum==1) 4 else 2,1,1))
    plot(x=xx,y=means,pch=20,col=cols,xlim=xlim,ylim=ylim,xaxt="n",yaxt="n",ylab="",cex=cex)
    arrows(xx,unlist(mins),xx,unlist(maxs),angle=90,code=3,length=0.06,col=cols)
    if (doublebars) {
      arrows(xx,unlist(mins-precs),xx,unlist(mins+precs),angle=90,code=3,length=0.03,col="red")
      arrows(xx,unlist(maxs-precs),xx,unlist(maxs+precs),angle=90,code=3,length=0.03,col="red")
    }

    staxlab(side=1,at=x,labels=colnames(data)[c(rep(F,numfacs),colfilter)],cex=staxcex)
    #axis(side=1,at=x,labels=colnames(data)[c(rep(F,numfacs),colfilter)],cex.axis=cexaxis)
    axis(side=2,cex.axis=cexaxis)
    title(ylab=sprintf("Mean (s), Workload %d",workload),cex.lab=cexaxis)

    # Draw a level at perf cpu cycles mean
    if (colfilter[5]) abline(h=parms[parms$workload==workload,numfacs+1+4*4],col="black")
  }

}
dev.off()
