library(plotrix)

data <- read.table("0-data.txt",header=T,colClasses="numeric")
numfacs <- 2
numdepvars <- ncol(data)-numfacs
workloads <- unique(data$workload)
numWorkloads <- length(workloads)

makeplot <- function(os) {

  # Allocate depvars to plots for each workload.
  # These plot numbers restart at 1 for each row.
  # The first row is for workload 0 and so on.
  # Even skipped workloads need a row here.
  colmap <- matrix(c(1,1,2,1,1,1,1,1,
		     1,2,3,1,2,2,2,2,
		     1,2,3,1,2,2,2,2,
		     1,2,1,1,1,1,1,1,
		     1,1,2,1,1,1,1,1),ncol=numdepvars,byrow=T)
  maxCols <- max(colmap)

  pdf(sprintf("violinplot-%d.pdf",os),pointsize=11)
  lwd <- 1.5
  cex <- 1
  cexaxis <- 1.1
  staxcex <- cexaxis * 2/3
  violscale <- 5/12
  layout(matrix(data=c(1,1,2,3,4,5,6,7,8,9,9,10,11,11,12),
    nrow=numWorkloads,ncol=maxCols,byrow=T),widths=c(2,4,1))

  for (workload in workloads) {
    plotsThisRow <- max(colmap[workload+1,])

    if (T) {
      # Find the largest range for any plot pertaining to this workload.
      range <- 0
      for (plotnum in 1:plotsThisRow) {
	colfilter <- colmap[workload+1,]==plotnum
	dfilter <- data[data$workload==workload & data$os==os,(numfacs+1):ncol(data)][colfilter]
	min <- min(dfilter)
	max <- max(dfilter)
	range <- max(range,max-min)
      }
      cat(sprintf("Workload %d max range %f\n",workload,range))
    } else {
      # Or just hard-code a fixed value to make all plots the same scale.
      range <- 1.439469
    }

    # Draw all of the plots for this workload to that scale.
    for (plotnum in 1:plotsThisRow) {
      colfilter <- colmap[workload+1,]==plotnum
      dfilter <- data[data$workload==workload & data$os==os,(numfacs+1):ncol(data)][colfilter]
      topx  <- sum(colfilter)
      x     <- 1:topx
      min <- min(dfilter)
      max <- max(dfilter)
      # The margin mysteriously gets *bigger* for wider plots.
      adj   <- if (topx==1) 1/1.75
	  else if (topx==2) 1/2
	  else if (topx<6)  1/2.5
	  else              1/3
      xlim  <- c(1-adj,topx+adj)
      adj   <- (range+min-max)/2
      ylim  <- if (adj<min) c(min-adj,max+adj) else c(0,range)
      par(mar=c(2.5,if (plotnum==1) 4 else 2,1,1))

      length <- sum(data$workload==workload & data$os==os)
      plot(x=NA,xlim=xlim,ylim=ylim,xaxt="n",yaxt="n",ylab="",cex=cex)
      for (xi in x) {
	di <- dfilter[,xi]
	if (var(di) > 0) {
	  violin <- density(di)
	  scale <- violscale/max(violin$y)
	  lines(xi+violin$y*scale,violin$x,col="blue",lwd=lwd)
	  lines(xi-violin$y*scale,violin$x,col="blue",lwd=lwd)
	} else {
	  lines(c(xi-violscale,xi+violscale),c(di[1],di[1]),col="blue",lwd=lwd)
	}
	# Combine with a scatter plot to show the gaps.
	points(rep(xi,length),di,pch=20,cex=cex,col="black")
      }

      staxlab(side=1,at=x,labels=colnames(data)[c(rep(F,numfacs),colfilter)],cex=staxcex)
      #axis(side=1,at=x,labels=colnames(data)[c(rep(F,numfacs),colfilter)],cex.axis=cexaxis)
      axis(side=2,cex.axis=cexaxis)
      title(ylab=sprintf("Workload %d (s)",workload),cex.lab=cexaxis)
    }

  }
  dev.off()
}

makeplot(0)
makeplot(1)
