// Compute the aliquot sequence beginning with the number supplied on the
// command line.

#include <iostream>
#include <vector>
#include <algorithm>
#include <gmpxx.h>
#include <assert.h>

// Sum of divisors function
mpz_class sigma (const mpz_class n_in) {
  assert (n_in >= 1);
  mpz_class n_out(0), limit(sqrt(n_in));
  for (mpz_class i(1); i<=limit; ++i)
    if (n_in % i == 0) {
      mpz_class j(n_in/i);
      n_out += i;
      if (j!=i) n_out += j;
    }
  return n_out;
}

// Convenience functions
inline bool contains (const std::vector<mpz_class> &v, const mpz_class &x) {
  return (std::find(v.begin(),v.end(),x) != v.end());
}
inline int index (const std::vector<mpz_class> &v, const mpz_class &x) {
  return (std::find(v.begin(),v.end(),x) - v.begin());
}

int main (int argc, char **argv) {
  if (argc != 2) {
    std::cerr << "Usage: aliquot N\n";
  } else {
    int itercount(1);
    mpz_class n(argv[1]);
    std::vector<mpz_class> seenvals;
    std::cout << itercount << " " << n << "\n";
    while (n>0 && !contains(seenvals,n)) {
      seenvals.push_back(n);
      n = sigma(n)-n;
      std::cout << ++itercount << " " << n << "\n";
    }
    if (n>0)
      std::cout << "Cycle of length " << itercount-index(seenvals,n)-1
                << " detected.\n";
  }
  return 0;
}
