# 2014-03-26 11:43

library(plotrix)

makeplot <- function(xid,fname) {

  # xid-based magic.
  numfacs <- if (xid==2) 3 else 2
  xtreatlabs <- if (xid==2) c(expression(fp %.% prf),expression(os %.% prf),expression(os %.% fp)) else c("fp","os")
  pdfwidth <- if (xid==2) 6.5 else 3.5
  omaright <- if (xid==2) 0 else 1.25

  pdf(fname,pointsize=11,width=pdfwidth,height=numfacs*2)

  data  <- read.table(sprintf("%d-data.txt",xid),header=T,colClasses="numeric")
  effects <- read.table(sprintf("%d-parms.txt",xid),header=T,colClasses="numeric")
  numdeps <- ncol(data) - numfacs

  # Keep only the original treatments.
  treats <- effects[apply(is.na(effects[,1:numfacs]),1,sum)==0,]

  # One page per experiment.
  # Columns:  depvars.
  # Rows:  factors (with the other two on the x axis).
  layout(matrix(data=seq(1,numdeps*numfacs),ncol=numdeps,byrow=T),
	 widths=rep(1,numdeps))
  par(oma=c(0,1.75,1,omaright))
  rowtitles <- colnames(data)[1:numfacs]
  coltitles <- colnames(data)[(numfacs+1):ncol(data)]

  # Use the same y axis scaling for both plots.
  if (xid==2) {
    means <- treats[,seq(numfacs+1,ncol(treats),4)]
    ymin <<- apply(means,2,min,na.rm=T)
    ymax <<- apply(means,2,max,na.rm=T)
  }

  for (fi in 1:numfacs) {

    # This is complicated.

    # Move factor fi to column 1.
    treats2 <- treats[,c(fi,(1:ncol(treats))[-fi])]

    # Enumerate the available values of x.
    # x is a product of the two other factors.
    # Residual ordering from parms should make the ordering here reasonable.
    xtreats <- unique(treats2[,2:numfacs,drop=F])

    # Assign x values based on matching xtreats.
    # There is probably a much better way to do this.
    x <- rep(NA,nrow(treats2))
    for (i in 1:length(x)) {
      for (j in 1:nrow(xtreats)) {
	match=T
	for (k in 1:ncol(xtreats)) {
	  if (xtreats[j,k]!=treats2[i,k+1]) {
	    match=F
	    break
	  }
	}
	if (match) {
	  x[i] = j
	  break
	}
      }
    }

    # Colorize some of the numbers.
    cols <- rep("black",nrow(treats))
    # "Dysfunctional" use cases (fp profiling but no fp).
    cols[xid==2 & treats[,2]==0 & treats[,3]==2] <- "red"

    # Create labels for the xtreats.
    # Hard-coded to numfacs.
    xlabs <- rep(NA,nrow(xtreats))
    for (i in 1:nrow(xtreats))
      xlabs[i] <- if (ncol(xtreats)==2) {
	  sprintf("%d%d",xtreats[i,1],xtreats[i,2])
	} else {
	  sprintf("%d",xtreats[i,1])
	}

    # With that mess done, plot all the depvars in a row.
    for (di in 1:numdeps) {
      means <- treats2[,di*4+numfacs-3]
      adj   <- 0.5
      xlim  <- c(1-adj,nrow(xtreats)+adj)
      xboxadj <- if (nrow(xtreats)<6) 0.25 else 0.33
      yboxadj <- (ymax[di] - ymin[di]) / 20
      ylim  <- c(ymin[di]-yboxadj,ymax[di]+yboxadj)
      par(mar=c(2,2,1,0.1))
      plot(NA,xlim=xlim,ylim=ylim,xaxt="n",cex=1,mgp=c(3,0.5,0))
      text(x=x,y=unlist(means),labels=treats2[,1],col=cols)
      axis(side=1,at=1:nrow(xtreats),labels=xlabs)
      if (di==1) {
	mtadj <- xlim[1] - (xlim[2]-xlim[1])/5
	mtext(xtreatlabs[fi],side=1,at=mtadj,line=(if (xid==2) 1.2 else 1),cex=0.85)
      }
      # Box drawing.
      xleft <- seq(1-xboxadj,nrow(xtreats))
      xright <- seq(1+xboxadj,nrow(xtreats)+xboxadj)
      ybottom <- rep(NA,nrow(xtreats))
      ytop <- rep(NA,nrow(xtreats))
      for (i in 1:length(x)) {
	xx <- x[i]
	if (is.na(ybottom[xx])) {
	  ybottom[xx] = means[i]
	} else if (!is.na(means[i]) && means[i]<ybottom[xx]) {
	  ybottom[xx] = means[i]
	}
	if (is.na(ytop[xx])) {
	  ytop[xx] = means[i]
	} else if (!is.na(means[i]) && means[i]>ytop[xx]) {
	  ytop[xx] = means[i]
	}
      }
      ybottom <- ybottom - yboxadj
      ytop <- ytop + yboxadj
      rect(xleft,ybottom,xright,ytop,border="black",col=NA)
    }
  }
  # Labels
  coladj = if (numfacs==3) c(0.07875,0.38,0.65375,0.96)
           else c(0,0.385,0.69,1.06)
  rowadj = if (numfacs==3) c(0.86,.5,0.17) else c(0.8,0.25)
  for (i in 1:numfacs) {
    mtext(text=colnames(treats)[i],side=2,line=0.5,outer=T,las=0,adj=rowadj[i])
  }
  for (i in 1:numdeps) {
    mtext(text=colnames(data)[i+numfacs],side=3,line=-0.5,outer=T,adj=coladj[i])
  }

  dev.off()
}

makeplot(2,"blockplot-main.pdf")
makeplot(5,"blockplot-showdown.pdf")
