#!/usr/bin/ruby

require 'GECS'

# Number of iterations expected
N = 5000

# Allow truncating if there are more?
TrunkOK = false

# Bootstrap or quick means
boot = true

# Retrieve perf data series as array of strings.
def getperfvar(filecontent)
  temp = filecontent.scan(/Event count \(approx.\): (\d*)/).flatten
  temp.slice!(N..-1) if TrunkOK && (temp.length > N)
  raise "Count mismatch" if temp.length != N
  temp
end

# Retrieve bash time data series as arrays of strings.
def getbashvar(filecontent,fieldnum)
  temp = filecontent.scan(/.*\n/).map(&:split).map{|x| x.fetch(fieldnum)}
  temp.slice!(N..-1) if TrunkOK && (temp.length > N)
  raise "Count mismatch" if temp.length != N
  temp
end

# Retrieve all data for one treatment.
# Convert all units to seconds and floats for simplicity.
# os=0,1 (14.0 vs. 14.1)
# fp=0,1 (frame pointers in app, no/yes)  prf=0,1,2 (no,dwarf,fp)
# There is no rep file when prf=0.
# depvars: bash-elapsed bash-user bash-sys perf-samples
# perf-samples is left nil when prf=0.
def gettreatment(os,fp,prf)
  perffile = File.open("14." + os.to_s + "/rep" + fp.to_s + prf.to_s + ".txt").read if prf>0
  bashfile = File.open("14." + os.to_s + "/log" + fp.to_s + prf.to_s + ".txt").read
  results = Array.new(4)
  for i in 0..2
    results[i] = getbashvar(bashfile,i).map(&:to_f)
  end
  if prf>0
    temp = getperfvar(perffile).map!(&:to_f)
    # Samples taken at 1000000 cycles; convert to seconds.
    temp.map!{|x| x/2992.5}
    results[3] = temp
  end
  results
end

bag = GECS.load("../1-bootdep/screening.gecs")

# Original
experiment = GECS::Experiment.new(2,
  ["os","fp","prf"],
  ["bash-elapsed", "bash-user", "bash-sys", "perf-samples"],
  "Three-factor experiment to screen callchain profiling overhead")
results = Hash.new
for os in 0..1
  for fp in 0..1
    for prf in 0..2
      results[GECS::Key.new(2,[os,fp,prf])] = gettreatment(os,fp,prf)
    end
  end
end
bag.experiments.push(experiment)  # 2
bag.data.merge!(results)

# purg1      Delete os=* fp=0 prf=2    (10 treatments)
experiment = GECS::Experiment.new(3,
  ["os","fp","prf"],
  ["bash-elapsed", "bash-user", "bash-sys", "perf-samples"],
  "Experiment 2 minus os=* fp=0 prf=2")
results = Hash.new
for os in 0..1
  for fp in 0..1
    for prf in 0..2
      results[GECS::Key.new(3,[os,fp,prf])] = gettreatment(os,fp,prf) unless fp==0 and prf==2
    end
  end
end
bag.experiments.push(experiment) # 3
bag.data.merge!(results)

# purg2      Delete os=* fp=* prf=2    (8 treatments)
experiment = GECS::Experiment.new(4,
  ["os","fp","prf"],
  ["bash-elapsed", "bash-user", "bash-sys", "perf-samples"],
  "Experiment 2 minus os=* fp=* prf=2")
results = Hash.new
for os in 0..1
  for fp in 0..1
    for prf in 0..1
      results[GECS::Key.new(4,[os,fp,prf])] = gettreatment(os,fp,prf)
    end
  end
end
bag.experiments.push(experiment) # 4
bag.data.merge!(results)

# showdown   Keep only os=* (fp=0 prf=1)|(fp=1 prf=2)   (4 treatments)
# Remove the prf factor as it becomes linked to fp.
experiment = GECS::Experiment.new(5,
  ["os","fp"],
  ["bash-elapsed", "bash-user", "bash-sys", "perf-samples"],
  "Experiment 2 keeping only os=* fp=0 prf=1 and fp=1 prf=2")
results = Hash.new
for os in 0..1
  # DWARF profiling with no fp
  results[GECS::Key.new(5,[os,0])] = gettreatment(os,0,1)
  # fp profiling with fp
  results[GECS::Key.new(5,[os,1])] = gettreatment(os,1,2)
end
bag.experiments.push(experiment) # 5
bag.data.merge!(results)

for id in 2..5
  if boot
    print "Bootstrapping experiment ", id, "\n"
    GECS.bootstrapMeans(bag,id,[0.001,0.001,0.0001,0.001])
  else
    GECS.quickMeans(bag,id)
  end
  print "Dumping experiment ", id, "\n"
  File.open(id.to_s + "-data.txt","w"){|fp| fp << GECS::dumpData(bag,id)}
  File.open(id.to_s + "-parms.txt","w"){|fp| fp << GECS::dumpParms(bag,id,boot)}
end
GECS.save("screening.gecs",bag)
