library(plotrix)

# If T then the estimated attained precision of the bootstrap intervals is
# used to put error bars on the error bars.  Once it has been established
# that the precision is below the plot resolution, this is no longer
# desirable.
doublebars <- F

cellmar <- c(2,2,1,0.5)

makeplot <- function() {
  expdesc <- c("Complete","Minus os=* fp=0 prf=2", "Minus os=* fp=* prf=2","Only os=* fp=0 prf=1 and fp=1 prf=2")

  for (xid in 2:5) {
    numfacs <- if (xid<5) 3 else 2
    data  <- read.table(sprintf("%d-data.txt",xid),header=T,colClasses="numeric")
    effects <- read.table(sprintf("%d-parms.txt",xid),header=T,colClasses="numeric")
    numdeps <- ncol(data) - numfacs

    # To avoid weird stuff, replace NA levels with the unused value -1.
    star <- -1
    for (i in 1:numfacs) effects[is.na(effects[,i]),i] <- star

    # One grid (page) per depvar.  Each gets its own scale so there is
    # no need to separate out bash.sys anymore.
    for (di in 1:numdeps) {

      fnam <- sprintf("%d-effects-%d.pdf",xid,di)
      pdfsize <- if (xid<5) 5 else 3.5
      pdf(fnam,pointsize=11,width=pdfsize,height=pdfsize)
      # Stupid scaling issues...
      oneScaleToRuleThemAll <- 1.2
      cex <- oneScaleToRuleThemAll * (if (xid<5) 1 else 0.8)
      cexaxis <- cex
      mtextcex <- oneScaleToRuleThemAll * 0.75

      # Find the range for the selected depvar.  All plots on a given page
      # have the same fixed range.
      ylim <- c(min(effects[,di*4+numfacs-2],na.rm=T),max(effects[,di*4+numfacs-1],na.rm=T))

      if (numfacs==3) {
        layout(matrix(data=c(1,4,5,
                             0,2,6,
                             0,0,3),ncol=3,byrow=T),widths=c(1,1,1))
      } else {
        layout(matrix(data=c(1,3,
                             0,2),ncol=2,byrow=T),widths=c(1,1))
      }

      par(oma=c(0,1.75,0,0))
      rowcoltitles <- colnames(data)[1:numfacs]

      # Main effects (one line each).
      for (fi in 1:numfacs) {
	parms <- effects[apply(effects[,1:numfacs]!=star,1,sum)==1 & effects[,fi]!=star,]
        x <- parms[,fi]
        cols <- "black"
        means <- parms[,di*4+numfacs-3]
        mins  <- parms[,di*4+numfacs-2]
        maxs  <- parms[,di*4+numfacs-1]
        precs <- parms[,di*4+numfacs]
	adj   <- 0.25
        xlim  <- c(min(x)-adj,max(x)+adj)
	par(mar=cellmar)
        plot(x=x,y=unlist(means),pch=20,col=cols,xlim=xlim,ylim=ylim,xaxt="n",yaxt="n",cex=cex)
        axis(side=1,at=x,labels=x,cex.axis=cexaxis)
        axis(side=2,cex.axis=cexaxis)
        mtext(rowcoltitles[fi],side=1,at=xlim[1]-(xlim[2]-xlim[1])/10,line=1,cex=mtextcex)
        lines(x=x,y=unlist(means))
        arrows(x,unlist(mins),x,unlist(maxs),
               angle=90,code=3,length=0.06,col=cols)
        if (doublebars) {
          arrows(x,unlist(mins-precs),x,unlist(mins+precs),angle=90,code=3,length=0.03,col="red")
          arrows(x,unlist(maxs-precs),x,unlist(maxs+precs),angle=90,code=3,length=0.03,col="red")
        }
      }

      # 2-way interactions (one line for each level of the row factor).
      for (rowfi in 1:(numfacs-1)) {
        for (colfi in (rowfi+1):numfacs) {
  	  parms <- effects[apply(effects[,1:numfacs]!=star,1,sum)==2
                           & effects[,rowfi]!=star & effects[,colfi]!=star,]
	  x <- parms[,colfi]
	  adj   <- 0.25
	  xlim  <- c(min(x)-adj,max(x)+adj)
          maxxi <- max(x)+1
	  par(mar=cellmar)
	  plot(NA,xlim=xlim,ylim=ylim,xaxt="n",yaxt="n",cex=cex)
          # One line at a time.  Color-match the means for the cases where
          # the lines are broken.
          rowlvls <- unique(parms[,rowfi])
          cols <- if (length(rowlvls)==2) c("blue","chocolate3") else c("blue","chocolate3","chartreuse3")
          legend <- rep(NA,length(rowlvls))
          meansum <- 0
          meanc <- 0
          for (i in 1:length(rowlvls)) {
            parms2 <- parms[parms[,rowfi]==rowlvls[i],]
            col <- cols[i]
	    means <- parms2[,di*4+numfacs-3]
	    mins  <- parms2[,di*4+numfacs-2]
	    maxs  <- parms2[,di*4+numfacs-1]
	    precs <- parms2[,di*4+numfacs]
            x <- parms2[,colfi]
            points(x=x,y=means,col=col,pch=20,cex=cex)
	    arrows(x,unlist(mins),x,unlist(maxs),
		   angle=90,code=3,length=0.06,col=col)
	    if (doublebars) {
	      arrows(x,unlist(mins-precs),x,unlist(mins+precs),angle=90,code=3,length=0.03,col="red")
	      arrows(x,unlist(maxs-precs),x,unlist(maxs+precs),angle=90,code=3,length=0.03,col="red")
	    }
            lines(x=x,y=means,col=col)
            legend[i] <- sprintf("%s=%d",colnames(data)[rowfi],rowlvls[i])
            if (!is.na(means[maxxi])) {
              meansum <- meansum + means[maxxi]
              meanc <- meanc+1
            }
          }
          # Dodgy (some lines might have more x's than others).
          axis(side=1,at=x,labels=x,cex.axis=cexaxis)
          axis(side=2,cex.axis=cexaxis)
          mtext(rowcoltitles[colfi],side=1,at=xlim[1]-(xlim[2]-xlim[1])/10,line=1,cex=mtextcex)
          plotloc <- if (meanc>0) meansum/meanc else 0
          legendloc <- if (abs(plotloc-ylim[1]) < abs(plotloc-ylim[2]))
            "topright" else "bottomright"
          legend(legendloc,col=cols,lty=1,legend=legend,cex=cex)
        }
      }

      # Labels
      #coladj = if (numfacs==3) c(0.17,0.5,0.85) else c(0.25,0.76)
      rowadj = if (numfacs==3) c(0.86,.5,0.17) else c(0.8,0.25)
      #for (i in 1:numfacs) {
      #  mtext(text=rowcoltitles[i],side=3,line=-0.5,outer=T,adj=coladj[i])
      #  mtext(text=rowcoltitles[i],side=2,line=0.5,outer=T,las=0,adj=rowadj[i])
      #}
      # title(main=sprintf("%s, %s",expdesc[xid],colnames(data)[numfacs+di]),line=2,outer=T)
      mtext(text="s",side=2,line=0.5,outer=T,las=0,adj=rowadj[1])
      dev.off()
    }
  }
}

makeplot()
