library(plotrix)

data <- read.table("data.txt",header=T,colClasses="numeric")

data0 <- data[data$BCa==0,]
data1 <- data[data$BCa==1,]
col0 <- "blue"
col1 <- "red"

# x axis is log2.
x <- 1:nrow(data0)

pdf("materiality-attcov.pdf",pointsize=11,width=7.5,height=5)
par(mar=c(4,4,0.1,0.1))
ymin <- min(data$attcov-data$attu)
ymax <- max(data$attcov+data$attu)
plot(x=x, y=data0$attcov, xlab="Size of subsample", ylab="Attained coverage of sample mean by 95 % interval", pch=16, lty=2, type="b", xaxt="n", ylim=c(ymin,ymax), col=col0)
arrows(x,data0$attcov-data0$attu,x,data0$attcov+data0$attu,
  angle=90,code=3,length=0.06,col=col0)
lines(x=x, y=data1$attcov, pch=16, lty=2, type="b", col=col1)
arrows(x,data1$attcov-data1$attu,x,data1$attcov+data1$attu,
  angle=90,code=3,length=0.06,col=col1)
axis(1,at=x,labels=data0$subsamp)
abline(h=0.95,lty=3)
legend("bottomright",col=c(col1,col0),legend=c("BCa interval","Original interval"),pch=16)
dev.off()

# Means, circles.  Medians, just add a line to the intervals.
pdf("materiality-intervals.pdf",pointsize=11,width=7.5,height=5)
par(mar=c(4,4,0.1,0.1))
ymin <- min(data$mean, data$q1)
ymax <- max(data$mean, data$q3)
plot(x=x, y=data0$mean, col=col0, xlab="Size of subsample", ylab="Width of 95 % interval (s)", pch=16, lty=2, type="b", xaxt="n", ylim=c(ymin,ymax))
lines(x=x, y=data1$mean, col=col1, pch=16, lty=2, type="b")

# Offset the lines slightly to prevent complete overlap.
fudge <- 0.01
arrows(x-fudge,data0$q1,x-fudge,data0$q3,
  angle=90,code=3,length=0.06,col=col0)
arrows(x+fudge,data1$q1,x+fudge,data1$q3,
  angle=90,code=3,length=0.06,col=col1)

medianwidth <- 0.1
segments(x0=x-fudge-medianwidth,y0=data0$q2,
         x1=x-fudge+medianwidth,y1=data0$q2,
         col=col0)
segments(x0=x+fudge-medianwidth,y0=data1$q2,
         x1=x+fudge+medianwidth,y1=data1$q2,
         col=col1)

abline(h=0.052683,lty=3)
legend("topright",legend=c("Mean BCa interval width","Mean original interval width","BCa interval width quartiles","Original interval width quartiles"),col=c("red","blue"),pch=c(16,16,NA,NA),lty=c(NA,NA,1,1))
axis(1,at=1:9,labels=data[1:9,2])
dev.off()
