numfacs <- 3
lwd <- 3
violscale <- 5/12

makeplot <- function() {
  data  <- read.table("2-data.txt",header=T,colClasses="numeric")
  numdeps <- ncol(data) - numfacs
  treats <- unique(data[,1:numfacs])
  numtreats <- nrow(treats)
  numreps <- nrow(data)/numtreats   # 5000

  # Create labels for the treatments.
  # Hard-coded to numfacs.
  xlabs <- rep(NA,nrow(treats))
  for (i in 1:nrow(treats))
    xlabs[i] <- sprintf("%d%d%d",treats[i,1],treats[i,2],treats[i,3])

  # The pdf is too much for Adobe Reader to render.
  png("violins.png",pointsize=22,width=1300,height=1200)
  layout(matrix(data=1:numdeps,ncol=1,byrow=T),widths=c(1))
  par(oma=c(0,1.75,0.15,0.15))
  for (di in 1:numdeps) {
    # We rely on the data being sorted by treatment.
    x <- rep(1:numtreats,each=numreps)

    fd <- data[,numfacs+di]
    ymin <- min(fd,na.rm=T)
    ymax <- max(fd,na.rm=T)
    adj <- 0.2
    xlim  <- c(1-adj,numtreats+adj)
    ylim  <- c(ymin,ymax)
    par(mar=c(2.6,4,0,0))

    # Scatter plots.
    plot(x=x,y=unlist(fd),pch=20,col="black",xlim=xlim,ylim=ylim,xaxt="n",yaxt="n",ylab="s",cex=1.5,cex.lab=1.5,bty="n")
    box(lwd=lwd)
    axis(side=1,at=1:nrow(treats),labels=xlabs,cex.axis=1.4,tck=-0.04,lwd=lwd)
    axis(side=2,cex.axis=1.4,tck=-0.04,lwd=lwd)
    mtext(expression(os %.% fp %.% prf),side=1,at=xlim[1]-0.85,line=1.25,cex=1)

    # Violin plots.
    for (i in 1:nrow(treats)) {
      # This is ridiculously, unforgivably slow.
      #treatdata <- data[apply(data,1,function(x) all(x[1:numfacs]==treats[i,])),numfacs+di]
      # Rely on residual ordering of data.  Very fast.
      treatdata <- data[(1+(i-1)*numreps):(i*numreps),numfacs+di]
      tdvar <- var(treatdata,na.rm=T)
      if (!is.na(tdvar)) {
        if (tdvar > 0) {
	  violin <- density(treatdata,na.rm=T)
	  scale <- violscale/max(violin$y)
	  lines(i+violin$y*scale,violin$x,col="blue",lwd=lwd)
	  lines(i-violin$y*scale,violin$x,col="blue",lwd=lwd)
	} else {
          vy <- max(treatdata,na.rm=T)
	  lines(c(i-violscale,i+violscale),c(vy,vy),col="blue",lwd=lwd)
	}
      }
    }

  }

  rowadj = c(0.95,0.655,0.3875,0.1)
  for (i in 1:numdeps) {
    mtext(text=colnames(data)[numfacs+i],side=2,line=0.5,outer=T,las=0,adj=rowadj[i])
  }
  dev.off()
}

makeplot()
